@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cmCTestSubmitCommand.cxx,v $
   Language:  C++
-  Date:      $Date: 2006-03-29 17:33:41 $
-  Version:   $Revision: 1.13 $
+  Date:      $Date: 2009-03-31 14:29:12 $
+  Version:   $Revision: 1.13.12.1 $
 
   Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -29,31 +29,66 @@ cmCTestGenericHandler* cmCTestSubmitCommand::InitializeHandler()
     = this->Makefile->GetDefinition("CTEST_DROP_LOCATION");
   const char* ctestTriggerSite
     = this->Makefile->GetDefinition("CTEST_TRIGGER_SITE");
+  bool ctestDropSiteCDash
+    = this->Makefile->IsOn("CTEST_DROP_SITE_CDASH");
 
   if ( !ctestDropMethod )
     {
     ctestDropMethod = "http";
     }
-  if ( !ctestDropSite )
+
+  if ( ctestDropSiteCDash )
     {
-    ctestDropSite = "public.kitware.com";
+    // drop site is a CDash server...
+    //
+    if ( !ctestDropSite )
+      {
+      // error: CDash requires CTEST_DROP_SITE definition
+      // in CTestConfig.cmake
+      }
+    if ( !ctestDropLocation )
+      {
+      // error: CDash requires CTEST_DROP_LOCATION definition
+      // in CTestConfig.cmake
+      }
     }
-  if ( !ctestDropLocation )
+  else
     {
-    ctestDropLocation = "/cgi-bin/HTTPUploadDartFile.cgi";
-    }
-  if ( !ctestTriggerSite )
-    {
-    ctestTriggerSite
-      = "http://public.kitware.com/cgi-bin/Submit-Random-TestingResults.cgi";
-    cmCTestLog(this->CTest, HANDLER_OUTPUT, "* Use default trigger site: "
-      << ctestTriggerSite << std::endl;);
+    // drop site is a *NOT* a CDash server...
+    //
+    // Keep all this code in case anybody out there is still
+    // using newer CMake with non-CDash servers
+    //
+    if ( !ctestDropSite )
+      {
+      ctestDropSite = "public.kitware.com";
+      }
+    if ( !ctestDropLocation )
+      {
+      ctestDropLocation = "/cgi-bin/HTTPUploadDartFile.cgi";
+      }
+    if ( !ctestTriggerSite )
+      {
+      ctestTriggerSite
+        = "http://public.kitware.com/cgi-bin/Submit-Random-TestingResults.cgi";
+      cmCTestLog(this->CTest, HANDLER_OUTPUT, "* Use default trigger site: "
+        << ctestTriggerSite << std::endl;);
+      }
     }
 
   this->CTest->SetCTestConfiguration("DropMethod",   ctestDropMethod);
   this->CTest->SetCTestConfiguration("DropSite",     ctestDropSite);
   this->CTest->SetCTestConfiguration("DropLocation", ctestDropLocation);
-  this->CTest->SetCTestConfiguration("TriggerSite",  ctestTriggerSite);
+
+  this->CTest->SetCTestConfiguration("IsCDash",
+    ctestDropSiteCDash ? "TRUE" : "FALSE");
+
+  // Only propagate TriggerSite for non-CDash projects:
+  //
+  if ( !ctestDropSiteCDash )
+    {
+    this->CTest->SetCTestConfiguration("TriggerSite",  ctestTriggerSite);
+    }
 
   this->CTest->SetCTestConfigurationFromCMakeVariable(this->Makefile,
     "DropSiteUser", "CTEST_DROP_SITE_USER");
