@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cmDependsFortran.cxx,v $
   Language:  C++
-  Date:      $Date: 2008-04-21 19:21:57 $
-  Version:   $Revision: 1.46.2.1 $
+  Date:      $Date: 2009-03-23 17:58:40 $
+  Version:   $Revision: 1.46.2.3 $
 
   Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -131,17 +131,28 @@ public:
 
 //----------------------------------------------------------------------------
 cmDependsFortran::cmDependsFortran():
-  IncludePath(0), PPDefinitions(0), Internal(0)
+  PPDefinitions(0), Internal(0)
 {
 }
 
 //----------------------------------------------------------------------------
 cmDependsFortran
-::cmDependsFortran(std::vector<std::string> const& includes,
-                   std::vector<std::string> const& definitions):
-  IncludePath(&includes),
+::cmDependsFortran(cmLocalGenerator* lg):
+  cmDepends(lg),
   Internal(new cmDependsFortranInternals)
 {
+  // Configure the include file search path.
+  this->SetIncludePathFromLanguage("Fortran");
+
+  // Get the list of definitions.
+  std::vector<std::string> definitions;
+  cmMakefile* mf = this->LocalGenerator->GetMakefile();
+  if(const char* c_defines =
+     mf->GetDefinition("CMAKE_TARGET_DEFINITIONS"))
+    {
+    cmSystemTools::ExpandListArgument(c_defines, definitions);
+    }
+
   // translate i.e. FOO=BAR to FOO and add it to the list of defined
   // preprocessor symbols
   for(std::vector<std::string>::const_iterator
@@ -178,11 +189,6 @@ bool cmDependsFortran::WriteDependencies(const char *src, const char *obj,
     cmSystemTools::Error("Cannot scan dependencies without an object file.");
     return false;
     }
-  if(!this->IncludePath)
-    {
-    cmSystemTools::Error("Cannot scan dependencies without an include path.");
-    return false;
-    }
 
   // Get the information object for this source.
   cmDependsFortranSourceInfo& info =
@@ -595,7 +601,7 @@ bool cmDependsFortran::FindModule(std::string const& name,
   // Search the include path for the module.
   std::string fullName;
   for(std::vector<std::string>::const_iterator i =
-        this->IncludePath->begin(); i != this->IncludePath->end(); ++i)
+        this->IncludePath.begin(); i != this->IncludePath.end(); ++i)
     {
     // Try the lower-case name.
     fullName = *i;
@@ -887,7 +893,7 @@ bool cmDependsFortran::FindIncludeFile(const char* dir,
 
     // Search the include path for the file.
     for(std::vector<std::string>::const_iterator i = 
-          this->IncludePath->begin(); i != this->IncludePath->end(); ++i)
+          this->IncludePath.begin(); i != this->IncludePath.end(); ++i)
       {
       fullName = *i;
       fullName += "/";
