@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cmDocumentation.h,v $
   Language:  C++
-  Date:      $Date: 2008-03-04 14:16:33 $
-  Version:   $Revision: 1.33 $
+  Date:      $Date: 2008-10-24 15:18:46 $
+  Version:   $Revision: 1.33.2.2 $
 
   Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -45,9 +45,12 @@ public:
    * Check command line arguments for documentation options.  Returns
    * true if documentation options are found, and false otherwise.
    * When true is returned, PrintRequestedDocumentation should be
-   * called.
+   * called.  exitOpt can be used for things like cmake -E, so that 
+   * all arguments after the -E are ignored and not searched for
+   * help arguments.
    */
-  bool CheckOptions(int argc, const char* const* argv);
+  bool CheckOptions(int argc, const char* const* argv, 
+                    const char* exitOpt =0);
   
   /**
    * Print help requested on the command line.  Call after
@@ -58,7 +61,7 @@ public:
   bool PrintRequestedDocumentation(std::ostream& os);
   
   /** Print help of the given type.  */
-  bool PrintDocumentation(Type ht, std::ostream& os);
+  bool PrintDocumentation(Type ht, std::ostream& os, const char* docname=0);
   
   /** Set the program name for standard document generation.  */
   void SetName(const char* name);
@@ -121,6 +124,7 @@ public:
 
 private:
   void SetForm(Form f);
+  void SetDocName(const char* docname);
 
   bool CreateSingleModule(const char* fname, 
                           const char* moduleName,
@@ -131,6 +135,8 @@ private:
   bool CreateCustomModulesSection();
   void CreateFullDocumentation();
 
+  void AddDocumentIntroToPrint(const char* intro[2]);
+
   bool PrintCopyright(std::ostream& os);
   bool PrintVersion(std::ostream& os);
   bool PrintDocumentationGeneric(std::ostream& os, const char *section);
@@ -154,9 +160,12 @@ private:
 
 
   const char* GetNameString() const;
+  const char* GetDocName(bool fallbackToNameString = true) const;
+  const char* GetDefaultDocName(Type ht) const;
   bool IsOption(const char* arg) const;
 
   std::string NameString;
+  std::string DocName;
   std::map<std::string,cmDocumentationSection*> AllSections;
   
   std::string SeeAlsoString;
