@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cmFindPackageCommand.h,v $
   Language:  C++
-  Date:      $Date: 2008-01-29 01:38:48 $
-  Version:   $Revision: 1.19 $
+  Date:      $Date: 2009-02-04 16:44:17 $
+  Version:   $Revision: 1.19.2.6 $
 
   Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -73,21 +73,31 @@ public:
 private:
   void AppendSuccessInformation();
   void AppendToProperty(const char* propertyName);
+  void SetModuleVariables(const std::string& components);
   bool FindModule(bool& found);
+  void AddFindDefinition(const char* var, const char* val);
+  void RestoreFindDefinitions();
   bool HandlePackageMode();
   void FindConfig();
   bool FindPrefixedConfig();
   bool FindFrameworkConfig();
   bool FindAppBundleConfig();
-  bool ReadListFile(const char* f);
+  enum PolicyScopeRule { NoPolicyScope, DoPolicyScope };
+  bool ReadListFile(const char* f, PolicyScopeRule psr);
   void StoreVersionFound();
 
-  void AddUserPath(std::string const& p);
   void ComputePrefixes();
+  void AddPrefixesCMakeEnvironment();
+  void AddPrefixesCMakeVariable();
+  void AddPrefixesSystemEnvironment();
+  void AddPrefixesBuilds();
+  void AddPrefixesCMakeSystemVariable();
+  void AddPrefixesUserGuess();
+  void AddPrefixesUserHints();
+  void ComputeFinalPrefixes();
   bool SearchDirectory(std::string const& dir);
   bool CheckDirectory(std::string const& dir);
   bool FindConfigFile(std::string const& dir, std::string& file);
-  bool FindConfigFileToLoad(std::string const& dir, std::string& file);
   bool CheckVersion(std::string const& config_file);
   bool CheckVersionFile(std::string const& version_file);
   bool SearchPrefix(std::string const& prefix);
@@ -96,6 +106,9 @@ private:
 
   friend class cmFindPackageFileList;
 
+  struct OriginalDef { bool exists; std::string value; };
+  std::map<cmStdString, OriginalDef> OriginalDefs;
+
   std::string CommandDocumentation;
   cmStdString Name;
   cmStdString Variable;
@@ -103,6 +116,7 @@ private:
   unsigned int VersionMajor;
   unsigned int VersionMinor;
   unsigned int VersionPatch;
+  unsigned int VersionTweak;
   unsigned int VersionCount;
   bool VersionExact;
   cmStdString FileFound;
@@ -110,6 +124,7 @@ private:
   unsigned int VersionFoundMajor;
   unsigned int VersionFoundMinor;
   unsigned int VersionFoundPatch;
+  unsigned int VersionFoundTweak;
   unsigned int VersionFoundCount;
   bool Quiet;
   bool Required;
@@ -117,10 +132,10 @@ private:
   bool NoModule;
   bool NoBuilds;
   bool DebugMode;
+  bool UseLib64Paths;
+  bool PolicyScope;
   std::vector<std::string> Names;
   std::vector<std::string> Configs;
-  std::vector<std::string> Prefixes;
-  std::vector<std::string> UserPaths;
 };
 
 #endif
