@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cmGetDirectoryPropertyCommand.cxx,v $
   Language:  C++
-  Date:      $Date: 2006/05/11 02:15:09 $
-  Version:   $Revision: 1.6.2.1 $
+  Date:      $Date: 2008-01-23 15:27:59 $
+  Version:   $Revision: 1.11 $
 
   Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -19,8 +19,8 @@
 #include "cmake.h"
 
 // cmGetDirectoryPropertyCommand
-bool cmGetDirectoryPropertyCommand::InitialPass(
-  std::vector<std::string> const& args)
+bool cmGetDirectoryPropertyCommand
+::InitialPass(std::vector<std::string> const& args, cmExecutionStatus &)
 {
   if(args.size() < 2 )
     {
@@ -28,7 +28,6 @@ bool cmGetDirectoryPropertyCommand::InitialPass(
     return false;
     }
   
-  std::vector<std::string>::size_type cc;
   std::vector<std::string>::const_iterator i = args.begin();
   std::string variable = *i;
   ++i;
@@ -53,6 +52,10 @@ bool cmGetDirectoryPropertyCommand::InitialPass(
       sd += "/";
       sd += *i;
       }
+
+    // The local generators are associated with collapsed paths.
+    sd = cmSystemTools::CollapseFullPath(sd.c_str());
+
     // lookup the makefile from the directory name
     cmLocalGenerator *lg = 
       this->Makefile->GetLocalGenerator()->GetGlobalGenerator()->
@@ -72,72 +75,7 @@ bool cmGetDirectoryPropertyCommand::InitialPass(
   // OK, now we have the directory to process, we just get the requested
   // information out of it
   
-  if ( *i == "VARIABLES" || *i == "CACHE_VARIABLES" )
-    {
-    int cacheonly = 0;
-    if ( *i == "CACHE_VARIABLES" )
-      {
-      cacheonly = 1;
-      }
-    std::vector<std::string> vars = dir->GetDefinitions(cacheonly);
-    for ( cc = 0; cc < vars.size(); cc ++ )
-      {
-      if ( cc > 0 )
-        {
-        output += ";";
-        }
-      output += vars[cc];
-      }
-    }
-  else if ( *i == "MACROS" )
-    {
-    dir->GetListOfMacros(output);
-    }
-  else if ( *i == "DEFINITIONS" )
-    {
-    output = dir->GetDefineFlags();
-    }
-  else if ( *i == "INCLUDE_DIRECTORIES" )
-    {
-    std::vector<std::string>::iterator it;
-    int first = 1;
-    cmOStringStream str;
-    for ( it = dir->GetIncludeDirectories().begin();
-      it != dir->GetIncludeDirectories().end();
-      ++ it )
-      {
-      if ( !first )
-        {
-        str << ";";
-        }
-      str << it->c_str();
-      first = 0;
-      }
-    output = str.str();
-    }
-  else if ( *i == "INCLUDE_REGULAR_EXPRESSION" )
-    {
-    output = dir->GetIncludeRegularExpression();
-    }
-  else if ( *i == "LINK_DIRECTORIES" )
-    {
-    std::vector<std::string>::iterator it;
-    int first = 1;
-    cmOStringStream str;
-    for ( it = dir->GetLinkDirectories().begin();
-      it != dir->GetLinkDirectories().end();
-      ++ it )
-      {
-      if ( !first )
-        {
-        str << ";";
-        }
-      str << it->c_str();
-      first = 0;
-      }
-    output = str.str();
-    }
-  else if ( *i == "DEFINITION" )
+  if ( *i == "DEFINITION" )
     {
     ++i;
     if (i == args.end())
@@ -147,20 +85,17 @@ bool cmGetDirectoryPropertyCommand::InitialPass(
       return false;
       }
     output = dir->GetSafeDefinition(i->c_str());
-    }
-  else
-    {
-    const char *prop = dir->GetProperty(i->c_str());
-    if (prop)
-      {
-      this->Makefile->AddDefinition(variable.c_str(), prop);
-      return true;
-      }
-    this->Makefile->AddDefinition(variable.c_str(), "");
+    this->Makefile->AddDefinition(variable.c_str(), output.c_str());
     return true;
     }
-  this->Makefile->AddDefinition(variable.c_str(), output.c_str());
-  
+
+  const char *prop = dir->GetProperty(i->c_str());
+  if (prop)
+    {
+    this->Makefile->AddDefinition(variable.c_str(), prop);
+    return true;
+    }
+  this->Makefile->AddDefinition(variable.c_str(), "");
   return true;
 }
 
