@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cmGetFilenameComponentCommand.cxx,v $
   Language:  C++
-  Date:      $Date: 2003/02/07 19:04:16 $
-  Version:   $Revision: 1.13 $
+  Date:      $Date: 2009-03-23 17:58:40 $
+  Version:   $Revision: 1.17.2.1 $
 
   Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -18,7 +18,8 @@
 #include "cmSystemTools.h"
 
 // cmGetFilenameComponentCommand
-bool cmGetFilenameComponentCommand::InitialPass(std::vector<std::string> const& args)
+bool cmGetFilenameComponentCommand
+::InitialPass(std::vector<std::string> const& args, cmExecutionStatus &)
 {
   if(args.size() < 3)
     {
@@ -30,7 +31,7 @@ bool cmGetFilenameComponentCommand::InitialPass(std::vector<std::string> const&
   // already, if so use that value
   if(args.size() == 4 && args[3] == "CACHE")
     {
-    const char* cacheValue = m_Makefile->GetDefinition(args[0].c_str());
+    const char* cacheValue = this->Makefile->GetDefinition(args[0].c_str());
     if(cacheValue && !cmSystemTools::IsNOTFOUND(cacheValue))
       {
       return true;
@@ -63,7 +64,8 @@ bool cmGetFilenameComponentCommand::InitialPass(std::vector<std::string> const&
           }
         }
       }
-    cmSystemTools::SplitProgramFromArgs(filename.c_str(), result, programArgs);
+    cmSystemTools::SplitProgramFromArgs(filename.c_str(), 
+                                        result, programArgs);
     }
   else if (args[2] == "EXT")
     {
@@ -73,9 +75,29 @@ bool cmGetFilenameComponentCommand::InitialPass(std::vector<std::string> const&
     {
     result = cmSystemTools::GetFilenameWithoutExtension(filename);
     }
-  else if (args[2] == "ABSOLUTE")
+  else if (args[2] == "ABSOLUTE" ||
+           args[2] == "REALPATH")
     {
+    // If the path given is relative evaluate it relative to the
+    // current source directory.
+    if(!cmSystemTools::FileIsFullPath(filename.c_str()))
+      {
+      std::string fname = this->Makefile->GetCurrentDirectory();
+      if(!fname.empty())
+        {
+        fname += "/";
+        fname += filename;
+        filename = fname;
+        }
+      }
+
+    // Collapse the path to its simplest form.
     result = cmSystemTools::CollapseFullPath(filename.c_str());
+    if(args[2] == "REALPATH")
+      {
+      // Resolve symlinks if possible
+      result = cmSystemTools::GetRealPath(filename.c_str());
+      }
     }
   else 
     {
@@ -88,25 +110,23 @@ bool cmGetFilenameComponentCommand::InitialPass(std::vector<std::string> const&
     {
     if(programArgs.size() && storeArgs.size())
       {
-      m_Makefile->AddCacheDefinition(storeArgs.c_str(),
-                                     programArgs.c_str(),
-                                     "",
-                                     args[2] == "PATH" ? cmCacheManager::FILEPATH
-                                     : cmCacheManager::STRING);
+      this->Makefile->AddCacheDefinition
+        (storeArgs.c_str(), programArgs.c_str(),
+         "", args[2] == "PATH" ? cmCacheManager::FILEPATH
+         : cmCacheManager::STRING);
       }
-    m_Makefile->AddCacheDefinition(args[0].c_str(),
-                                   result.c_str(),
-                                   "",
-                                   args[2] == "PATH" ? cmCacheManager::FILEPATH
-                                                     : cmCacheManager::STRING);
+    this->Makefile->AddCacheDefinition
+      (args[0].c_str(), result.c_str(), "",
+       args[2] == "PATH" ? cmCacheManager::FILEPATH
+       : cmCacheManager::STRING);
     }
   else 
     {
     if(programArgs.size() && storeArgs.size())
       {
-      m_Makefile->AddDefinition(storeArgs.c_str(), programArgs.c_str());
+      this->Makefile->AddDefinition(storeArgs.c_str(), programArgs.c_str());
       }
-    m_Makefile->AddDefinition(args[0].c_str(), result.c_str());
+    this->Makefile->AddDefinition(args[0].c_str(), result.c_str());
     }
 
   return true;
