@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cmGetFilenameComponentCommand.h,v $
   Language:  C++
-  Date:      $Date: 2006/05/11 02:15:09 $
-  Version:   $Revision: 1.11.6.1 $
+  Date:      $Date: 2009-03-23 17:58:40 $
+  Version:   $Revision: 1.14.2.1 $
 
   Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -40,7 +40,8 @@ public:
    * This is called when the command is first encountered in
    * the CMakeLists.txt file.
    */
-  virtual bool InitialPass(std::vector<std::string> const& args);
+  virtual bool InitialPass(std::vector<std::string> const& args,
+                           cmExecutionStatus &status);
 
   /**
    * This determines if the command is invoked when in script mode.
@@ -50,7 +51,7 @@ public:
   /**
    * The name of the command as specified in CMakeList.txt.
    */
-  virtual const char* GetName() { return "GET_FILENAME_COMPONENT";}
+  virtual const char* GetName() { return "get_filename_component";}
 
   /**
    * Succinct documentation.
@@ -66,17 +67,18 @@ public:
   virtual const char* GetFullDocumentation()
     {
     return
-      "  GET_FILENAME_COMPONENT(VarName FileName\n"
-      "                         PATH|ABSOLUTE|NAME|EXT|NAME_WE\n"
+      "  get_filename_component(VarName FileName\n"
+      "                         PATH|ABSOLUTE|NAME|EXT|NAME_WE|REALPATH\n"
       "                         [CACHE])\n"
       "Set VarName to be the path (PATH), file name (NAME), file "
       "extension (EXT), file name without extension (NAME_WE) of FileName, "
-      "or the full absolute (ABSOLUTE) file name without symlinks.  "
+      "the full path (ABSOLUTE), or the full path with all symlinks "
+      "resolved (REALPATH).  "
       "Note that the path is converted to Unix slashes format and has no "
       "trailing slashes. The longest file extension is always considered. "
       "If the optional CACHE argument is specified, the result variable is "
       "added to the cache.\n"
-      "  GET_FILENAME_COMPONENT(VarName FileName\n"
+      "  get_filename_component(VarName FileName\n"
       "                         PROGRAM [PROGRAM_ARGS ArgVar]\n"
       "                         [CACHE])\n"
       "The program in FileName will be found in the system search path or "
