@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cmGlobalVisualStudio6Generator.h,v $
   Language:  C++
-  Date:      $Date: 2006/06/30 17:48:43 $
-  Version:   $Revision: 1.25.2.2 $
+  Date:      $Date: 2007-06-28 13:09:26 $
+  Version:   $Revision: 1.29 $
 
   Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -17,7 +17,7 @@
 #ifndef cmGlobalVisualStudio6Generator_h
 #define cmGlobalVisualStudio6Generator_h
 
-#include "cmGlobalGenerator.h"
+#include "cmGlobalVisualStudioGenerator.h"
 
 class cmTarget;
 
@@ -26,7 +26,7 @@ class cmTarget;
  *
  * cmGlobalVisualStudio6Generator manages UNIX build process for a tree
  */
-class cmGlobalVisualStudio6Generator : public cmGlobalGenerator
+class cmGlobalVisualStudio6Generator : public cmGlobalVisualStudioGenerator
 {
 public:
   cmGlobalVisualStudio6Generator();
@@ -49,7 +49,7 @@ public:
    * extension, pthreads, byte order etc.  
    */
   virtual void EnableLanguage(std::vector<std::string>const& languages, 
-                              cmMakefile *);
+                              cmMakefile *, bool optional);
 
   /**
    * Try running cmake and building a file. This is used for dynalically
