@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cmGlobalVisualStudio7Generator.cxx,v $
   Language:  C++
-  Date:      $Date: 2008-05-01 16:35:39 $
-  Version:   $Revision: 1.99.2.1 $
+  Date:      $Date: 2009-02-04 16:44:17 $
+  Version:   $Revision: 1.99.2.4 $
 
   Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -253,15 +253,7 @@ void cmGlobalVisualStudio7Generator::AddAllBuildDepends(
     cmTarget* t = const_cast<cmTarget*>(*ot);
     if(!this->IsExcluded(root, *t))
       {
-      if (t->GetType() == cmTarget::UTILITY ||
-          t->GetType() == cmTarget::GLOBAL_TARGET)
-        {
-        target->AddUtility(t->GetName());
-        }
-      else
-        {
-        target->AddLinkLibrary(t->GetName(),cmTarget::GENERAL);
-        }
+      target->AddUtility(t->GetName());
       }
     }
 }
@@ -269,14 +261,14 @@ void cmGlobalVisualStudio7Generator::AddAllBuildDepends(
 void cmGlobalVisualStudio7Generator::WriteTargetConfigurations(
   std::ostream& fout, 
   cmLocalGenerator* root,
-  cmGlobalGenerator::TargetDependSet& projectTargets)
+  OrderedTargetDependSet const& projectTargets)
 {
   // loop over again and write out configurations for each target
   // in the solution
-  for(cmGlobalGenerator::TargetDependSet::iterator tt =
+  for(OrderedTargetDependSet::const_iterator tt =
         projectTargets.begin(); tt != projectTargets.end(); ++tt)
     {
-    cmTarget* target = const_cast<cmTarget*>(*tt);
+    cmTarget* target = *tt;
     if (strncmp(target->GetName(), "INCLUDE_EXTERNAL_MSPROJECT", 26) == 0)
       {
       cmCustomCommand cc = target->GetPostBuildCommands()[0];
@@ -304,16 +296,16 @@ void cmGlobalVisualStudio7Generator::WriteTargetConfigurations(
 void cmGlobalVisualStudio7Generator::WriteTargetsToSolution(
     std::ostream& fout,
     cmLocalGenerator* root,
-    cmGlobalGenerator::TargetDependSet& projectTargets,
+    OrderedTargetDependSet const& projectTargets,
     cmGlobalGenerator::TargetDependSet& originalTargets
     )
 {
   std::string rootdir = root->GetMakefile()->GetStartOutputDirectory();
   rootdir += "/";
-  for(cmGlobalGenerator::TargetDependSet::iterator tt =
+  for(OrderedTargetDependSet::const_iterator tt =
         projectTargets.begin(); tt != projectTargets.end(); ++tt)
     {
-    cmTarget* target = const_cast<cmTarget*>(*tt);
+    cmTarget* target = *tt;
     cmMakefile* mf = target->GetMakefile();
     // look for the all_build rule and add depends to all
     // of the original targets (none that were "pulled" into this project)
@@ -370,13 +362,13 @@ void cmGlobalVisualStudio7Generator::WriteTargetsToSolution(
 
 void cmGlobalVisualStudio7Generator::WriteTargetDepends(
     std::ostream& fout,
-    cmGlobalGenerator::TargetDependSet& projectTargets
+    OrderedTargetDependSet const& projectTargets
     )
 {
-  for(cmGlobalGenerator::TargetDependSet::iterator tt =
+  for(OrderedTargetDependSet::const_iterator tt =
         projectTargets.begin(); tt != projectTargets.end(); ++tt)
     {
-    cmTarget* target = const_cast<cmTarget*>(*tt);
+    cmTarget* target = *tt;
     cmMakefile* mf = target->GetMakefile();
     if (strncmp(target->GetName(), "INCLUDE_EXTERNAL_MSPROJECT", 26) == 0)
       {
@@ -434,7 +426,9 @@ void cmGlobalVisualStudio7Generator
   this->GetTargetSets(projectTargets,
                       originalTargets,
                       root, generators);
-  this->WriteTargetsToSolution(fout, root, projectTargets, originalTargets);
+  OrderedTargetDependSet orderedProjectTargets(projectTargets);
+  this->WriteTargetsToSolution(fout, root, orderedProjectTargets,
+                               originalTargets);
   // Write out the configurations information for the solution
   fout << "Global\n"
        << "\tGlobalSection(SolutionConfiguration) = preSolution\n";
@@ -449,12 +443,12 @@ void cmGlobalVisualStudio7Generator
   fout << "\tEndGlobalSection\n";
   // Write out project(target) depends 
   fout << "\tGlobalSection(ProjectDependencies) = postSolution\n";
-  this->WriteTargetDepends(fout, projectTargets);
+  this->WriteTargetDepends(fout, orderedProjectTargets);
   fout << "\tEndGlobalSection\n";
 
   // Write out the configurations for all the targets in the project
   fout << "\tGlobalSection(ProjectConfiguration) = postSolution\n";
-  this->WriteTargetConfigurations(fout, root, projectTargets);
+  this->WriteTargetConfigurations(fout, root, orderedProjectTargets);
   fout << "\tEndGlobalSection\n";
 
   // Write the footer for the SLN file
@@ -486,7 +480,8 @@ void cmGlobalVisualStudio7Generator::WriteProject(std::ostream& fout,
 { 
    // check to see if this is a fortran build
   const char* ext = ".vcproj";
-  const char* project = "Project(\"{8BC9CEB8-8B4A-11D0-8D11-00A0C91BC942}\") = \"";
+  const char* project =
+    "Project(\"{8BC9CEB8-8B4A-11D0-8D11-00A0C91BC942}\") = \"";
   if(this->TargetIsFortranOnly(target))
     {
     ext = ".vfproj";
@@ -729,6 +724,34 @@ bool cmGlobalVisualStudio7Generator::IsPartOfDefaultBuild(const char* project,
   return true;
 }
 
+//----------------------------------------------------------------------------
+bool
+cmGlobalVisualStudio7Generator::TargetCompare
+::operator()(cmTarget const* l, cmTarget const* r)
+{
+  // Make sure ALL_BUILD is first so it is the default active project.
+  if(strcmp(r->GetName(), "ALL_BUILD") == 0)
+    {
+    return false;
+    }
+  if(strcmp(l->GetName(), "ALL_BUILD") == 0)
+    {
+    return true;
+    }
+  return strcmp(l->GetName(), r->GetName()) < 0;
+}
+
+//----------------------------------------------------------------------------
+cmGlobalVisualStudio7Generator::OrderedTargetDependSet
+::OrderedTargetDependSet(cmGlobalGenerator::TargetDependSet const& targets)
+{
+  for(cmGlobalGenerator::TargetDependSet::const_iterator ti =
+        targets.begin(); ti != targets.end(); ++ti)
+    {
+    this->insert(*ti);
+    }
+}
+
 //----------------------------------------------------------------------------
 static cmVS7FlagTable cmVS7ExtraFlagTable[] =
 {
@@ -743,6 +766,7 @@ static cmVS7FlagTable cmVS7ExtraFlagTable[] =
    cmVS7FlagTable::UserValueIgnored | cmVS7FlagTable::Continue},
   {"PrecompiledHeaderThrough", "Yu", "Precompiled Header Name", "",
    cmVS7FlagTable::UserValueRequired},
+  {"WholeProgramOptimization", "LTCG", "WholeProgramOptimization", "TRUE", 0},
 
   // Exception handling mode.  If no entries match, it will be FALSE.
   {"ExceptionHandling", "GX", "enable c++ exceptions", "TRUE", 0},
