@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cmGlobalXCodeGenerator.h,v $
   Language:  C++
-  Date:      $Date: 2008-01-14 14:20:57 $
-  Version:   $Revision: 1.52 $
+  Date:      $Date: 2009-03-23 17:58:41 $
+  Version:   $Revision: 1.52.2.3 $
 
   Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -103,6 +103,8 @@ private:
                             cmXCodeObject* frameworkBuildPhase,
                             cmTarget& cmtarget);
 
+  std::string ComputeInfoPListLocation(cmTarget& target);
+
   void AddCommandsToBuildPhase(cmXCodeObject* buildphase,
                                cmTarget& target,
                                std::vector<cmCustomCommand> 
@@ -150,11 +152,14 @@ private:
   virtual void  WriteXCodePBXProj(std::ostream& fout,
                                   cmLocalGenerator* root,
                                   std::vector<cmLocalGenerator*>& generators);
+  cmXCodeObject* CreateXCodeFileReference(cmSourceFile* sf,
+                                          cmTarget& cmtarget);
   cmXCodeObject* CreateXCodeSourceFile(cmLocalGenerator* gen, 
                                        cmSourceFile* sf,
                                        cmTarget& cmtarget);
   void CreateXCodeTargets(cmLocalGenerator* gen, 
                           std::vector<cmXCodeObject*>&);
+  bool IsHeaderFile(cmSourceFile*);
   void AddDependTarget(cmXCodeObject* target,
                        cmXCodeObject* dependTarget);
   void CreateXCodeDependHackTarget(std::vector<cmXCodeObject*>& targets);
