@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cmMakefileTargetGenerator.cxx,v $
   Language:  C++
-  Date:      $Date: 2008-04-23 02:05:40 $
-  Version:   $Revision: 1.93.2.3 $
+  Date:      $Date: 2009-03-23 17:58:48 $
+  Version:   $Revision: 1.93.2.8 $
 
   Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -24,6 +24,7 @@
 #include "cmSourceFile.h"
 #include "cmTarget.h"
 #include "cmake.h"
+#include "cmComputeLinkInformation.h"
 
 #include "cmMakefileExecutableTargetGenerator.h"
 #include "cmMakefileLibraryTargetGenerator.h"
@@ -267,8 +268,7 @@ void cmMakefileTargetGenerator::WriteTargetLanguageFlags()
     // Add the export symbol definition for shared library objects.
     if(const char* exportMacro = this->Target->GetExportMacro())
       {
-      flags += "-D";
-      flags += exportMacro;
+      this->LocalGenerator->AppendDefines(defines, exportMacro, lang);
       }
 
     // Add preprocessor definitions for this target and configuration.
@@ -566,7 +566,7 @@ cmMakefileTargetGenerator
   if(this->LocalGenerator->UseRelativePaths)
     {
     sourceFile = this->Convert(sourceFile.c_str(),
-                               cmLocalGenerator::HOME_OUTPUT);
+                               cmLocalGenerator::START_OUTPUT);
     }
   sourceFile = this->Convert(sourceFile.c_str(),
                              cmLocalGenerator::NONE,
@@ -613,7 +613,7 @@ cmMakefileTargetGenerator
   this->LocalGenerator->CreateCDCommand
     (compileCommands,
      this->Makefile->GetStartOutputDirectory(),
-     this->Makefile->GetHomeOutputDirectory());
+     cmLocalGenerator::HOME_OUTPUT);
   commands.insert(commands.end(),
                   compileCommands.begin(), compileCommands.end());
 
@@ -724,7 +724,7 @@ cmMakefileTargetGenerator
         this->LocalGenerator->CreateCDCommand
           (preprocessCommands,
            this->Makefile->GetStartOutputDirectory(),
-           this->Makefile->GetHomeOutputDirectory());
+           cmLocalGenerator::HOME_OUTPUT);
         commands.insert(commands.end(),
                         preprocessCommands.begin(),
                         preprocessCommands.end());
@@ -780,7 +780,7 @@ cmMakefileTargetGenerator
         this->LocalGenerator->CreateCDCommand
           (assemblyCommands,
            this->Makefile->GetStartOutputDirectory(),
-           this->Makefile->GetHomeOutputDirectory());
+           cmLocalGenerator::HOME_OUTPUT);
         commands.insert(commands.end(),
                         assemblyCommands.begin(),
                         assemblyCommands.end());
@@ -894,7 +894,7 @@ void cmMakefileTargetGenerator::WriteTargetCleanRules()
   this->LocalGenerator->CreateCDCommand
     (commands,
      this->Makefile->GetStartOutputDirectory(),
-     this->Makefile->GetHomeOutputDirectory());
+     cmLocalGenerator::HOME_OUTPUT);
 
   // Write the rule.
   this->LocalGenerator->WriteMakeRule(*this->BuildFileStream, 0,
@@ -933,8 +933,10 @@ void cmMakefileTargetGenerator::WriteTargetDependRules()
           this->MultipleOutputPairs.begin();
         pi != this->MultipleOutputPairs.end(); ++pi)
       {
-      *this->InfoFileStream << "  \"" << pi->first << "\" \""
-                            << pi->second << "\"\n";
+      *this->InfoFileStream
+        << "  " << this->LocalGenerator->EscapeForCMake(pi->first.c_str())
+        << " "  << this->LocalGenerator->EscapeForCMake(pi->second.c_str())
+        << "\n";
       }
     *this->InfoFileStream << "  )\n\n";
     }
@@ -945,16 +947,16 @@ void cmMakefileTargetGenerator::WriteTargetDependRules()
     << "\n"
     << "# Targets to which this target links.\n"
     << "SET(CMAKE_TARGET_LINKED_INFO_FILES\n";
-  cmGlobalGenerator* gg = this->GlobalGenerator;
   std::set<cmTarget const*> emitted;
-  cmTarget::LinkLibraryVectorType const& libs =
-    this->Target->GetLinkLibraries();
-  for(cmTarget::LinkLibraryVectorType::const_iterator j = libs.begin();
-      j != libs.end(); ++j)
+  const char* cfg = this->LocalGenerator->ConfigurationName.c_str();
+  if(cmComputeLinkInformation* cli = this->Target->GetLinkInformation(cfg))
     {
-    if(cmTarget const* linkee = gg->FindTarget(0, j->first.c_str()))
+    cmComputeLinkInformation::ItemVector const& items = cli->GetItems();
+    for(cmComputeLinkInformation::ItemVector::const_iterator
+          i = items.begin(); i != items.end(); ++i)
       {
-      if(emitted.insert(linkee).second)
+      cmTarget const* linkee = i->Target;
+      if(linkee && !linkee->IsImported() && emitted.insert(linkee).second)
         {
         cmMakefile* mf = linkee->GetMakefile();
         cmLocalGenerator* lg = mf->GetLocalGenerator();
@@ -1028,8 +1030,11 @@ void cmMakefileTargetGenerator::WriteTargetDependRules()
                           cmLocalGenerator::FULL, cmLocalGenerator::SHELL)
          << " "
          << this->Convert(this->InfoFileNameFull.c_str(),
-                          cmLocalGenerator::FULL, cmLocalGenerator::SHELL)
-         << " --color=$(COLOR)";
+                          cmLocalGenerator::FULL, cmLocalGenerator::SHELL);
+  if(this->LocalGenerator->GetColorMakefile())
+    {
+    depCmd << " --color=$(COLOR)";
+    }
   commands.push_back(depCmd.str());
 
   // Make sure all custom command outputs in this target are built.
@@ -1113,16 +1118,16 @@ void cmMakefileTargetGenerator
       ->AppendEcho(commands, comment.c_str(),
                    cmLocalUnixMakefileGenerator3::EchoGenerate);
     }
+  // Below we need to skip over the echo and progress commands.
+  unsigned int skip = static_cast<unsigned int>(commands.size());
+
+  // Now append the actual user-specified commands.
   this->LocalGenerator->AppendCustomCommand(commands, cc);
 
   // Collect the dependencies.
   std::vector<std::string> depends;
   this->LocalGenerator->AppendCustomDepend(depends, cc);
 
-  // Add a dependency on the rule file itself.
-  this->LocalGenerator->AppendRuleDepend(depends,
-                                         this->BuildFileNameFull.c_str());
-
   // Check whether we need to bother checking for a symbolic output.
   bool need_symbolic = this->GlobalGenerator->GetNeedSymbolicMark();
 
@@ -1141,6 +1146,14 @@ void cmMakefileTargetGenerator
   this->LocalGenerator->WriteMakeRule(*this->BuildFileStream, 0,
                                       o->c_str(), depends, commands,
                                       symbolic);
+
+  // If the rule has changed make sure the output is rebuilt.
+  if(!symbolic)
+    {
+    this->GlobalGenerator->AddRuleHash(cc.GetOutputs(),
+                                       commands.begin()+skip,
+                                       commands.end());
+    }
   }
 
   // Write rules to drive building any outputs beyond the first.
@@ -1441,7 +1454,9 @@ std::string cmMakefileTargetGenerator::GetFrameworkFlags()
     if(emitted.insert(*i).second)
       {
       flags += "-F";
-      flags += this->LocalGenerator->ConvertToOutputForExisting(i->c_str());
+      flags += this->Convert(i->c_str(),
+                             cmLocalGenerator::START_OUTPUT,
+                             cmLocalGenerator::SHELL, true);
       flags += " ";
       }
     }
@@ -1458,48 +1473,16 @@ void cmMakefileTargetGenerator
     {
     return;
     }
-  // Compute which library configuration to link.
-  cmTarget::LinkLibraryType linkType = cmTarget::OPTIMIZED;
-  if(cmSystemTools::UpperCase(
-       this->LocalGenerator->ConfigurationName.c_str()) == "DEBUG")
-    {
-    linkType = cmTarget::DEBUG;
-    }
-  // Keep track of dependencies already listed.
-  std::set<cmStdString> emitted;
-
-  // A target should not depend on itself.
-  emitted.insert(this->Target->GetName());
 
   // Loop over all library dependencies.
-  const cmTarget::LinkLibraryVectorType& tlibs =
-    this->Target->GetLinkLibraries();
-  for(cmTarget::LinkLibraryVectorType::const_iterator lib = tlibs.begin();
-      lib != tlibs.end(); ++lib)
+  const char* cfg = this->LocalGenerator->ConfigurationName.c_str();
+  if(cmComputeLinkInformation* cli = this->Target->GetLinkInformation(cfg))
     {
-    // skip the library if it is not general and the link type
-    // does not match the current target
-    if(lib->second != cmTarget::GENERAL &&
-       lib->second != linkType)
+    std::vector<std::string> const& libDeps = cli->GetDepends();
+    for(std::vector<std::string>::const_iterator j = libDeps.begin();
+        j != libDeps.end(); ++j)
       {
-      continue;
-      }
-       
-    // Don't emit the same library twice for this target.
-    if(emitted.insert(lib->first).second)
-      {
-      // Depend on other CMake targets.
-      if(cmTarget* tgt =
-         this->GlobalGenerator->FindTarget(0, lib->first.c_str()))
-        {
-        const char* config = this->LocalGenerator->ConfigurationName.c_str();
-        depends.push_back(tgt->GetFullPath(config, false));
-        }
-      // depend on full path libs as well
-      else if(cmSystemTools::FileIsFullPath(lib->first.c_str()))
-        {
-        depends.push_back(lib->first.c_str());
-        }
+      depends.push_back(*j);
       }
     }
 }
@@ -1641,6 +1624,65 @@ cmMakefileTargetGenerator
   return responseFileName;
 }
 
+//----------------------------------------------------------------------------
+void
+cmMakefileTargetGenerator
+::CreateObjectLists(bool useLinkScript, bool useArchiveRules,
+                    bool useResponseFile, std::string& buildObjs,
+                    std::vector<std::string>& makefile_depends)
+{
+  std::string variableName;
+  std::string variableNameExternal;
+  this->WriteObjectsVariable(variableName, variableNameExternal);
+  if(useResponseFile)
+    {
+    // MSVC response files cannot exceed 128K.
+    std::string::size_type const responseFileLimit = 131000;
+
+    // Construct the individual object list strings.
+    std::vector<std::string> object_strings;
+    this->WriteObjectsStrings(object_strings, responseFileLimit);
+
+    // Write a response file for each string.
+    const char* sep = "";
+    for(unsigned int i = 0; i < object_strings.size(); ++i)
+      {
+      // Number the response files.
+      char rsp[32];
+      sprintf(rsp, "objects%u.rsp", i+1);
+
+      // Create this response file.
+      std::string objects_rsp =
+        this->CreateResponseFile(rsp, object_strings[i], makefile_depends);
+
+      // Separate from previous response file references.
+      buildObjs += sep;
+      sep = " ";
+
+      // Reference the response file.
+      buildObjs += "@";
+      buildObjs += this->Convert(objects_rsp.c_str(),
+                                 cmLocalGenerator::NONE,
+                                 cmLocalGenerator::SHELL);
+      }
+    }
+  else if(useLinkScript)
+    {
+    if(!useArchiveRules)
+      {
+      this->WriteObjectsString(buildObjs);
+      }
+    }
+  else
+    {
+    buildObjs = "$(";
+    buildObjs += variableName;
+    buildObjs += ") $(";
+    buildObjs += variableNameExternal;
+    buildObjs += ")";
+    }
+}
+
 //----------------------------------------------------------------------------
 const char* cmMakefileTargetGenerator::GetFortranModuleDirectory()
 {
