@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cmSetTargetPropertiesCommand.h,v $
   Language:  C++
-  Date:      $Date: 2006/05/14 19:22:43 $
-  Version:   $Revision: 1.21.2.2 $
+  Date:      $Date: 2008-10-24 15:18:54 $
+  Version:   $Revision: 1.31.2.1 $
 
   Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -31,12 +31,13 @@ public:
    * This is called when the command is first encountered in
    * the input file.
    */
-  virtual bool InitialPass(std::vector<std::string> const& args);
+  virtual bool InitialPass(std::vector<std::string> const& args,
+                           cmExecutionStatus &status);
 
   /**
    * The name of the command as specified in CMakeList.txt.
    */
-  virtual const char* GetName() { return "SET_TARGET_PROPERTIES";}
+  virtual const char* GetName() { return "set_target_properties";}  
 
   /**
    * Succinct documentation.
@@ -46,13 +47,20 @@ public:
     return "Targets can have properties that affect how they are built.";
     }
   
+  /**
+   *  Used by this command and cmSetPropertiesCommand
+   */
+  static bool SetOneTarget(const char *tname, 
+                           std::vector<std::string> &propertyPairs, 
+                           cmMakefile *mf);
+
   /**
    * Longer documentation.
    */
   virtual const char* GetFullDocumentation()
     {
       return
-        "  SET_TARGET_PROPERTIES(target1 target2 ...\n"
+        "  set_target_properties(target1 target2 ...\n"
         "                        PROPERTIES prop1 value1\n"
         "                        prop2 value2 ...)\n"
         "Set properties on a target. The syntax for the command is to "
@@ -108,10 +116,17 @@ public:
         "the same version number. "
         "For executables VERSION can be used to specify the build version. "
         "When building or installing appropriate symlinks are created if "
-        "the platform supports symlinks.\n"
+        "the platform supports symlinks. "
+        "For shared libraries and executables on Windows the VERSION "
+        "attribute is parsed to extract a \"major.minor\" version number. "
+        "These numbers are used as the image version of the binary. "
+        "\n"
         "There are a few properties used to specify RPATH rules. "
         "INSTALL_RPATH is a semicolon-separated list specifying the rpath "
         "to use in installed targets (for platforms that support it). "
+        "INSTALL_RPATH_USE_LINK_PATH is a boolean that if set to true will "
+        "append directories in the linker search path and outside the "
+        "project to the INSTALL_RPATH. "
         "SKIP_BUILD_RPATH is a boolean specifying whether to skip automatic "
         "generation of an rpath allowing the target to run from the "
         "build tree. "
@@ -122,18 +137,33 @@ public:
         "directory portion of the \"install_name\" field of shared libraries "
         "on Mac OSX to use in the installed targets. "
         "When the target is created the values of "
-        "the variables CMAKE_INSTALL_RPATH, CMAKE_SKIP_BUILD_RPATH, "
+        "the variables CMAKE_INSTALL_RPATH, "
+        "CMAKE_INSTALL_RPATH_USE_LINK_PATH, CMAKE_SKIP_BUILD_RPATH, "
         "CMAKE_BUILD_WITH_INSTALL_RPATH, and CMAKE_INSTALL_NAME_DIR "
         "are used to initialize these properties.\n"
         "PROJECT_LABEL can be used to change the name of "
         "the target in an IDE like visual studio.  VS_KEYWORD can be set "
         "to change the visual studio keyword, for example QT integration "
         "works better if this is set to Qt4VSv1.0.\n"
+        "VS_SCC_PROJECTNAME, VS_SCC_LOCALPATH, VS_SCC_PROVIDER can be set "
+        "to add support for source control bindings in a  Visual Studio "
+        "project file.\n"
+        "When a library is built CMake by default generates code to remove "
+        "any existing library using all possible names.  This is needed "
+        "to support libraries that switch between STATIC and SHARED by "
+        "a user option.  However when using OUTPUT_NAME to build a static "
+        "and shared library of the same name using different logical target "
+        "names the two targets will remove each other's files.  This can be "
+        "prevented by setting the CLEAN_DIRECT_OUTPUT property to 1.\n"
         "The PRE_INSTALL_SCRIPT and POST_INSTALL_SCRIPT properties are the "
         "old way to specify CMake scripts to run before and after "
         "installing a target.  They are used only when the old "
         "INSTALL_TARGETS command is used to install the target.  Use the "
         "INSTALL command instead."
+        "\n"
+        "The EXCLUDE_FROM_DEFAULT_BUILD property is used by the visual "
+        "studio generators.  If it is set to 1 the target will not be "
+        "part of the default build when you select \"Build Solution\"."
         ;
     }
   
